#!/usr/bin/python

# This code is in the public domain.

# Step 2: as fetch3.py (see there for comments what's going on)
# but now we're querying async, in parallel

import sys
import time

import vohelper
import pyvo

QUERIES = [
  ("twomass", "http://dc.zah.uni-heidelberg.de/tap",
    """SELECT TOP 1000000 raj2000, dej2000, jmag, hmag, kmag 
        FROM twomass.data
        WHERE 1=CONTAINS(
                      POINT('ICRS', raj2000, dej2000),
                      CIRCLE('ICRS', {ra}, {dec}, {radius}))"""),
  # limiting wise matches since both vizier and astropy's 
  # VOTable parser are lame in some sense
  ("allwise", "http://tapvizier.u-strasbg.fr/TAPVizieR/tap",
    """SELECT raj2000, dej2000, w1mag, w2mag, w3mag, w4mag
        FROM "II/328/allwise"
        WHERE w1mag<14 AND
          1=CONTAINS(
                      POINT('ICRS', raj2000, dej2000),
                      CIRCLE('ICRS', {ra}, {dec}, {radius}))"""),
  ("sdss", "http://gea.esac.esa.int/tap-server/tap",
    """SELECT ra, dec, u_mag, g_mag, r_mag, i_mag, z_mag
        FROM gaiadr1.sdssdr9_original_valid
        WHERE 1=CONTAINS(
                      POINT('ICRS', ra, dec),
                      CIRCLE('ICRS', {ra}, {dec}, {radius}))"""),]

def main():
  if len(sys.argv)!=4:
    ra, dec, radius = 30, 10, 0.20
  else:
    ra, dec, radius = [float(a) for a in sys.argv[1:]]

  jobs = set()
  for short_name, access_url, query in QUERIES:
    # in async, you first create a job:
    job = pyvo.dal.TAPService(access_url).submit_job(
      query.format(**locals()), maxrec=9000000)
    # then start it. This immediately returns.
    job.run()
    # we keep note of the jobs we started -- we'll watch them later.
    jobs.add((short_name, job))

  with vohelper.SAMP_conn() as conn:
    topcat_id = vohelper.find_client(conn, "topcat")

    # now watch jobs until they return, then take them off the watch list
    # and send their result
    while jobs:
      # we do the list(.) so we can remove jobs with impunity
      for short_name, job in list(jobs):
        # async jobs are in phases; they're done (or failed) when
        # they're neither queued nor executing.
        print short_name, job.phase
        if job.phase not in ('QUEUED', 'EXECUTING'):
          jobs.remove((short_name, job))
          vohelper.send_table_to(
            conn, 
            topcat_id, 
            # this is how you get the result from a finished job
            job.fetch_result().table, 
            short_name)
          # be a good citizen: clean up your job (it'll be cleaned up
          # eventually anyway, but that might take a while)
          job.delete()

      # wait a bit before doing the next round of polling
      time.sleep(0.5)


if __name__=="__main__":
  main()
